@@ -9,7 +9,37 @@ import cpu
 import device
 import hypervisor
 
+PROFILE_VERSION = 0
+
+SYS_CLASS_DMI = "/sys/class/dmi/id"
+
+class Singleton(type):
+	def __init__(cls, name, bases, dict):
+		super(Singleton, cls).__init__(name, bases, dict)
+		cls.instance = None
+
+	def __call__(cls, *args, **kw):
+		if cls.instance is None:
+			cls.instance = super(Singleton, cls).__call__(*args, **kw)
+
+		return cls.instance
+
+
+def read_from_file(filename):
+	"""
+		Read all data from filename.
+	"""
+	if not os.path.exists(filename):
+		return
+
+	try:
+		with open(filename) as f:
+			return f.read().strip()
+	except IOError:
+		pass
+
 class System(object):
+	__metaclass__ = Singleton
 
 	def __init__(self):
 		# find all devices
@@ -19,32 +49,48 @@ class System(object):
 		self.hypervisor = hypervisor.Hypervisor()
 
 	def profile(self):
-		p = {
-			"public_id" : self.public_id,
-			"private_id" : self.private_id,
-			
-			"arch" : self.arch,
-			"language" : self.language,
-			"release" : self.release,
-			"memory" : self.memory,
-			"kernel" : self.kernel,
-			"root_size" : self.root_size,
-			"devices" : [],
+		p = {}
+		p["system"] = {
+			# System information
+			"model"  : self.model,
+			"vendor" : self.vendor,
 
+			# Indicator if the system is running in a
+			# virtual environment.
 			"virtual" : self.virtual,
-			}
 			
+			# System language
+			"language" : self.language,
+
+			# Release information
+			"release" : self.release,
+			"kernel_release" : self.kernel_release,
+
+			"memory" : self.memory,
+			"root_size" : self.root_size,
+		}
+
+		p["devices"] = []
 		for device in self.devices:
-			p["devices"].append({
+			d = {
 				"subsystem" : device.subsystem.lower(), 
 				"vendor" : device.vendor.lower(), 
 				"model" : device.model.lower(), 
 				"deviceclass" : device.deviceclass
-			})
-		
+			}
+
+			# PCI devices provide subsystem information, USB don't.
+			if d["subsystem"] == "pci":
+				d["sub_model"] = device.sub_model
+				d["sub_vendor"] = device.sub_vendor
+
+			p["devices"].append(d)
+
 		p["cpu"] = {
+			"arch" : self.arch,
 			"vendor" : self.cpu.vendor,
 			"model" : self.cpu.model,
+			"model_string" : self.cpu.model_string,
 			"stepping" : self.cpu.stepping,
 			"flags" : self.cpu.flags,
 			"bogomips" : self.cpu.bogomips,
@@ -60,7 +106,17 @@ class System(object):
 				"vendor" : self.hypervisor.vendor,
 			}
 
-		return p
+		return {
+			# Profile version
+			"profile_version" : PROFILE_VERSION,
+
+			# Identification and authorization codes
+			"public_id" : self.public_id,
+			"private_id" : self.private_id,
+
+			# Actual profile data
+			"profile" : p,
+		}
 				
 		
 	@property
@@ -83,7 +139,11 @@ class System(object):
 
 			No one could ever guess this without access to the host.
 		"""
-		return hashlib.sha1("%s" % reversed(self._unique_id)).hexdigest()
+		private_id = ""
+		for i in reversed(self._unique_id):
+			private_id += i
+
+		return hashlib.sha1(private_id).hexdigest()
 
 	@property
 	def _unique_id(self):
@@ -110,17 +170,40 @@ class System(object):
 
 	@property
 	def release(self):
-		with open("/etc/system-release", "r") as f:
-			return f.read().strip()
-		
+		return read_from_file("/etc/system-release")
+
+	@property
+	def bios_vendor(self):
+		return read_from_file("/sys/class/dmi/id/bios_vendor")
+
+	@property
+	def vendor(self):
+		ret = None
+		for file in ("chassis_vendor", "board_vendor", "sys_vendor",):
+			ret = read_from_file(os.path.join(SYS_CLASS_DMI, file))
+			if ret:
+				break
+
+		return ret
+
+	@property
+	def model(self):
+		ret = None
+		for file in ("chassis_model", "board_model", "product_name",):
+			ret = read_from_file(os.path.join(SYS_CLASS_DMI, file))
+			if ret:
+				break
+
+		return ret
+
 	@property
 	def memory(self):
 		with open("/proc/meminfo", "r") as f:
 			firstline = f.readline().strip()
-			return firstline.split()[1]
+			return int(firstline.split()[1])
 
 	@property
-	def kernel(self):
+	def kernel_release(self):
 		return os.uname()[2]
 
 	@property
@@ -164,6 +247,7 @@ if __name__ == "__main__":
 	print s.arch
 	print s.language
 	print s.release
+	print s.bios_vendor
 	print s.memory
 	print s.kernel
 	print s.root_disk
